/* rectangle_test.c -- test suite for rectangle.h and rectangle.c */

/* rj@elilabs.com Sun Nov 10 22:14:04 CST 2002 */

#include <stdio.h> /* NULL */
#include <string.h>
#include "rectangle.h"

#define SUCCESS \
printf("%s:%d SUCCESS\n", __FILE__, __LINE__); \
exit(0)

#define FAILURE \
printf("%s:%d FAILURE\n", __FILE__, __LINE__); \
exit(1)

#define MSG(msg) printf("%s:%d %s\n", __FILE__, __LINE__, msg)

void exit(int return_code);


int main(int nargs, char* argv[]) {
  rectangle* R1;
  rectangle* R2;
  rectangle* R;
  rectangle* L;
  int i;

  MSG("constructing R1");
  R1 = construct_rectangle(10, 20, 30, 40, "alpha");
  show_rectangle(R1);

  MSG("constructing R2");
  R2 = construct_rectangle(5, 15, 30, 17, "beta");
  show_rectangle(R2);

  MSG("expecting FALSE on next line");
  printf("point_is_inside_rectangle_P(0, 1, R1) = %s\n", point_is_inside_rectangle_P(0, 1, R1) ? "TRUE" : "FALSE");
  MSG("expecting TRUE on next line");
  printf("point_is_inside_rectangle_P(15, 15, R2) = %s\n", point_is_inside_rectangle_P(15, 15, R2) ? "TRUE" : "FALSE");

  destruct_rectangle(R1);
  destruct_rectangle(R2);

  MSG("Testing insert_rectangle_before");
  L = construct_rectangle(0, 0, 0, 0, "gamma");
  show_rectangle(L);
  for (i = 0; i < 3; i++) {
    R = construct_rectangle(i, i, i + 10, i + 10, "delta");
    show_rectangle(R);
    insert_rectangle_before(L, R);				 /* insert R before L */
    show_rectangle(L);
  }

  MSG("Testing remove_rectangle_from_list");
  for (R = L->next ; R != L; R = L->next) {
    show_rectangle(R);
    remove_rectangle_from_list(R);
    show_rectangle(R);
    destruct_rectangle(R);
  }

  MSG("Testing add_rectangle_to_list");
  L = construct_empty_list_of_rectangles("bag");
  show_rectangle(L);
  for (i = 0; i < 10; i++) {

    char space[10];						 /* generate a unique name */
    char *gensym = space;
    strcpy(gensym, "gensym_A");
    gensym[7] += (char)i;

    R = construct_rectangle(i, i, i + 10, i + 10, gensym);
    show_rectangle(R);
    add_rectangle_to_list(L, R);
    show_rectangle(L);
  }

  MSG("Testing remove_named_rectangle_from_list");
  R = remove_named_rectangle_from_list(L, "C");
  if (R != NULL) {
    show_rectangle(R);
  } else {
    printf("R = NULL\n");
  }
  show_rectangle(L);

  SUCCESS;
}
